<?php //$Id: block_activeusers.php,v 1.1 2008/05/12 12:41:09 rashan Exp $

class block_activeusers extends block_base {

	function init() {
		$this->title = get_string('activeusers', 'block_activeusers');
		$this->cron = 1;
		$this->version = 2008051201;
	}

	function has_config() {return true;}
	
	function applicable_formats() {return array('all' => true);}

	function instance_allow_config() {return true;}

	function instance_allow_multiple() {return true;}
	
	// Overide the parent class method just to update stats if needed
	function instance_config_save($data,$pinned=false) {
		// Update stats immediately
		if(isset($data->updatestats)){
			$this->update_stats($this->instance);
		}
		return parent::instance_config_save($data,$pinned);
    }

	function get_content() {
		global $CFG, $COURSE;
		if ($this->content !== NULL) {
			return $this->content;
		}

		if (!empty($this->config->title)) {
			$this->title = $this->config->title;
		}
		// Deside the number of Users to display
		$numusers = 10;
		if(!empty($this->config->numberofusers)){
			$numusers = $this->config->numberofusers;
		}
		if (empty($this->config->showrank)) {
			$this->config->showrank=0;
		}

		$this->content = new stdClass;

		// Get the list of users
		$topusers = get_records_select("block_activeusers","blockid={$this->instance->id}","rank DESC","userid,rank",0,$numusers);
		if($topusers){
			foreach ($topusers as $user){
				$userdata = get_record_select("user","id=$user->userid");
				$this->content->text .= '<div class="user"><a href="'.$CFG->wwwroot.'/user/view.php?id='.$user->userid.'&amp;course='.$COURSE->id.'" title="Active User">';
				$this->content->text .= print_user_picture($user->userid, $COURSE->id, 1, 16, true, false, '', false);
				$this->content->text .= fullname($userdata);

				if($this->config->showrank){
					$this->content->text .= ":: ".round($user->rank,2);
				}
				$this->content->text .="</a></div><br/>";
			}
		}

		$this->content->footer = '';
		return $this->content;
	}

	/**
	 * Update statistics of all the blocks via cron
	 */
	function cron(){
		$instanceid = get_record_select("block","name='activeusers'","id");
		if(!empty($instanceid)){
			//Get all the instances
			$instances = get_records_select("block_instance","blockid='".$instanceid->id."'");
			// Update statistics for each block
			foreach($instances as $instance){												
				$this->update_stats($instance,true);
			}
		}
	}
	
	/**
	 * Update statistics of a given block instance
	 */
	function update_stats($instance,$cron=false){
		global $CFG;
		
		$instance->config = unserialize(base64_decode($instance->configdata));
		// Deside the number of Users to display
		$numusers = 10;
		if(!empty($instance->config->numberofusers)){
			$numusers = $instance->config->numberofusers;
		}
		if (empty($instance->config->havestart)) {
			$instance->config->havestart=0;
		}
		if (empty($instance->config->haveend)) {
			$instance->config->haveend=0;
		}
		if (empty($instance->config->tmpltype)) {
			$instance->config->tmpltype='none';
		}

		// Test for templates and decide on date range
		$startdate = 0;
		$enddate = time();
		switch ($instance->config->tmpltype)
		{
			case "thisyear":
				$startdate = strtotime(date("Y")."-01-01");
				break;
			case "thismonth":
				$startdate = strtotime(date("Y-M"));
				break;
			case "thisweek":
				$startdate = strtotime(date("Y")."-01-01") + ((date("W")-1)*7*24*3600);
				break;
			case "today":
				$startdate = strtotime(date("Y-M")) + ((date("j")-1)*24*3600);
				break;
			default:
				// Also incase of 'none'
				if($instance->config->havestart){
					$tmpconfig = $instance->config->startdate;
					$startdate =  strtotime($tmpconfig['year']."-".$tmpconfig['month']."-".$tmpconfig['day']);
				}

				if($instance->config->haveend){
					$tmpconfig = $instance->config->enddate;
					$enddate =  strtotime($tmpconfig['year']."-".$tmpconfig['month']."-".$tmpconfig['day']);
				}
		}

		if($cron){
			$timeperiod = $enddate - $startdate;
			$lastupdated = get_record_select("block_activeusers","blockid={$instance->id}","MAX(lastupdated) AS lastupdated");
			$lastupdated = $lastupdated->lastupdated;
			$todaystart = strtotime(date("Y-M")) + 3600*((date("j")-1)*24*3600);
			// Time period is less than or equal to a day, update every updatetime_minor hours
			if($timeperiod < 86401 ){
				if((time() - $lastupdated) < (get_config('block_activeusers_updatetime_major'))){
					return;
				}
			}else{ // For longer time periods, update at updatetime_major
				$nextupdate = $todaystart + 3600*get_config('block_activeusers_updatetime_major');
				$debug->nextupdate = $nextupdate;
				if(($nextupdate - time() ) > 0 || (time() - $lastupdated) < 86400 )return; // Update time not reached OR updated recently				
			}					
		}
			
		// Filter results acording to the start date and end date (will be applyied later)
		$timefilter = " time >= $startdate AND time <= $enddate";
		// Filter results acording to the context; site level or course level
		if ($instance->pageid == SITEID) {  // Site-level
			// Get the userids with number of log records for site
			// TODO: Filter based on roles
			$sql_filterlogs = "SELECT userid, count(userid) AS count FROM {$CFG->prefix}log WHERE $timefilter GROUP BY userid";
		} else { // Course Level
			// Filter results based on current course (will be applied later)
			$coursefilter = " course = '".$instance->pageid."'";
			// Get the current course context
			$context=get_context_instance(CONTEXT_COURSE,$instance->pageid);
			
			// Build the initial sql_filter
			$sql_filterlogs = "SELECT ra.userid, count(ra.userid) AS count FROM {$CFG->prefix}log l,{$CFG->prefix}role_assignments ra
								 WHERE l.userid=ra.userid AND ra.hidden=0 AND ra.contextid=$context->id";
			// Get logs related to given roles 
			if($roles=$this->get_display_roles($instance->config)){
				$sql_filterlogs .= " AND (";
				foreach($roles as $role){
					$sql_filterlogs .= "roleid = $role->id OR ";
				}
				$sql_filterlogs .= "0 )";
			}
			// Apply the course and time filters
			$sql_filterlogs .= " AND $coursefilter AND $timefilter GROUP BY userid";
		}

		// Get the given number of top users based on filters above
		$sql = "SELECT u.id, u.username, u.firstname, u.lastname, u.picture, ac.count
                FROM {$CFG->prefix}user u, ($sql_filterlogs ORDER BY count DESC LIMIT $numusers) ac
                WHERE ac.userid = u.id";		
		$topusers = get_records_sql($sql,0,$numusers);

		if($topusers){
			// Get the average
			$sql = "SELECT AVG(count) AS avg FROM ($sql_filterlogs) a";
			$tmp = get_record_sql($sql);
			$avg = $tmp->avg;

			// Get the standard deviation
			$sql = "SELECT std(count) AS std FROM ($sql_filterlogs) a";
			$tmp = get_record_sql($sql);
			$stdev = $tmp->std;

			// Workarround for division by Zero error occured when there is only one user in the system (very very rare!)
			$stdev /= 1.0000;
			$stdev = $stdev ? $stdev: 1;

			// Delete any existing data
			delete_records("block_activeusers","blockid",$instance->id);
			// Insert new data
			foreach ($topusers as $user){
				$entry = new object();
				$entry->blockid = $instance->id;
				$entry->rank = ( $user->count - $avg )/$stdev;
				$entry->userid = $user->id;
				$entry->lastupdated = time();
				insert_record("block_activeusers",$entry);
			}	
		}
	}

	/**
 	* Prints the start date selector
 	*/
	function print_start_date_selector() {

		for ($i=1; $i<=31; $i++) {
			$days[$i] = $i;
		}
		for ($i=1; $i<=12; $i++) {
			$months[$i] = userdate(gmmktime(12,0,0,$i,15,2000), "%B");
		}
		for ($i=ACTIVESTARTYEAR; $i<=ACTIVEENDYEAR; $i++) {
			$years[$i] = $i;
		}
		return choose_from_menu($days, "startday",$this->config->startday, '', '', '0',false,$this->config->enablestartdate)
		.choose_from_menu($months, "startmonth", $this->config->startmonth,  '', '', '0',false,$this->config->enablestartdate)
		.choose_from_menu($years,  "startyear",  $this->config->startyear, '', '', '0',false,$this->config->enablestartdate);
	}

	/**
 	* Prints the end date selector
 	*/	
	function print_end_date_selector($status=false) {

		for ($i=1; $i<=31; $i++) {
			$days[$i] = $i;
		}
		for ($i=1; $i<=12; $i++) {
			$months[$i] = userdate(gmmktime(12,0,0,$i,15,2000), "%B");
		}
		for ($i=ACTIVESTARTYEAR; $i<=ACTIVEENDYEAR; $i++) {
			$years[$i] = $i;
		}
		return choose_from_menu($days, "endday",$this->config->endday, '', '', '0',false,$status)
		.choose_from_menu($months, "endmonth", $this->config->endmonth,  '', '', '0',false,$status)
		.choose_from_menu($years,  "endyear",  $this->config->endyear, '', '', '0',false,$status);

	}

	/**
 	* Get the role ids as an array
 	*/	
	static function get_display_roles($configs){
		$ids = Array();
		if($configs){
			foreach($configs as $key=>$value){
				if(ereg("(role_)([0-9]*)",$key,$regs)){
					$ids[]->id = $regs[2];
				}
			}
		}
		return $ids;
	}

}
?>